'use strict';

var _SETTINGS = {};
var defaultStyle = void 0;
var defaultHTML = void 0;
var _defaults = {
  distThreshold: 60,
  distMax: 80,
  distReload: 50,
  bodyOffset: 20,
  mainElement: 'body',
  triggerElement: 'body',
  ptrElement: '.ptr',
  classPrefix: 'ptr--',
  cssProp: 'min-height',
  containerClassName: '',
  boxClassName: '',
  contentClassName: '',
  textClassName: '',
  instructionsPullToRefresh: 'Pull down to refresh',
  instructionsReleaseToRefresh: 'Release to refresh',
  instructionsRefreshing: 'Refreshing',
  refreshTimeout: 500,
  onInit: function onInit() {},
  onRefresh: function onRefresh() {
    return location.reload();
  },
  resistanceFunction: function resistanceFunction(t) {
    return Math.min(1, t / 2.5);
  }
};

var pullStartY = null;
var pullMoveY = null;
var dist = 0;
var distResisted = 0;

var _state = 'pending';
var _setup = false;
var _enable = false;
var _timeout = void 0;

function _update() {
  var _SETTINGS2 = _SETTINGS,
      classPrefix = _SETTINGS2.classPrefix,
      ptrElement = _SETTINGS2.ptrElement,
      instructionsRefreshing = _SETTINGS2.instructionsRefreshing,
      instructionsPullToRefresh = _SETTINGS2.instructionsPullToRefresh,
      instructionsReleaseToRefresh = _SETTINGS2.instructionsReleaseToRefresh;


  var textEl = ptrElement.querySelector('.' + classPrefix + 'text');

  if (_state === 'releasing') {
    textEl.innerHTML = instructionsReleaseToRefresh;
  }

  if (_state === 'pulling' || _state === 'pending') {
    textEl.innerHTML = instructionsPullToRefresh;
  }

  if (_state === 'refreshing') {
    textEl.innerHTML = instructionsRefreshing;
  }
}

function _setupEvents() {
  function onReset() {
    var _SETTINGS3 = _SETTINGS,
        cssProp = _SETTINGS3.cssProp,
        ptrElement = _SETTINGS3.ptrElement,
        classPrefix = _SETTINGS3.classPrefix;


    ptrElement.classList.remove(classPrefix + 'refresh');
    ptrElement.style[cssProp] = '0px';

    _state = 'pending';
  }

  function _onTouchStart(e) {
    var _SETTINGS4 = _SETTINGS,
        triggerElement = _SETTINGS4.triggerElement;


    if (!window.scrollY) {
      pullStartY = e.touches[0].screenY;
    }

    if (_state !== 'pending') {
      return;
    }

    clearTimeout(_timeout);

    _enable = triggerElement.contains(e.target);
    _state = 'pending';
    _update();
  }

  function _onTouchMove(e) {
    var _SETTINGS5 = _SETTINGS,
        ptrElement = _SETTINGS5.ptrElement,
        resistanceFunction = _SETTINGS5.resistanceFunction,
        distMax = _SETTINGS5.distMax,
        distThreshold = _SETTINGS5.distThreshold,
        cssProp = _SETTINGS5.cssProp,
        classPrefix = _SETTINGS5.classPrefix;


    if (!pullStartY) {
      if (!window.scrollY) {
        pullStartY = e.touches[0].screenY;
      }
    } else {
      pullMoveY = e.touches[0].screenY;
    }

    if (!_enable || _state === 'refreshing') {
      if (!window.scrollY && pullStartY < pullMoveY) {
        e.preventDefault();
      }

      return;
    }

    if (_state === 'pending') {
      ptrElement.classList.add(classPrefix + 'pull');
      _state = 'pulling';
      _update();
    }

    if (pullStartY && pullMoveY) {
      dist = pullMoveY - pullStartY;
    }

    if (dist > 0) {
      e.preventDefault();

      ptrElement.style[cssProp] = distResisted + 'px';

      distResisted = resistanceFunction(dist / distThreshold) * Math.min(distMax, dist);

      if (_state === 'pulling' && distResisted > distThreshold) {
        ptrElement.classList.add(classPrefix + 'release');
        _state = 'releasing';
        _update();
      }

      if (_state === 'releasing' && distResisted < distThreshold) {
        ptrElement.classList.remove(classPrefix + 'release');
        _state = 'pulling';
        _update();
      }
    }
  }

  function _onTouchEnd() {
    var _SETTINGS6 = _SETTINGS,
        ptrElement = _SETTINGS6.ptrElement,
        onRefresh = _SETTINGS6.onRefresh,
        refreshTimeout = _SETTINGS6.refreshTimeout,
        distThreshold = _SETTINGS6.distThreshold,
        distReload = _SETTINGS6.distReload,
        cssProp = _SETTINGS6.cssProp,
        classPrefix = _SETTINGS6.classPrefix;


    if (_state === 'releasing' && distResisted > distThreshold) {
      _state = 'refreshing';

      ptrElement.style[cssProp] = distReload + 'px';
      ptrElement.classList.add(classPrefix + 'refresh');

      _timeout = setTimeout(function () {
        var retval = onRefresh(onReset);

        if (retval && typeof retval.then === 'function') {
          retval.then(function () {
            return onReset();
          });
        }

        if (!retval && !onRefresh.length) {
          onReset();
        }
      }, refreshTimeout);
    } else {
      if (_state === 'refreshing') {
        return;
      }

      ptrElement.style[cssProp] = '0px';

      _state = 'pending';
    }

    _update();

    ptrElement.classList.remove(classPrefix + 'release');
    ptrElement.classList.remove(classPrefix + 'pull');

    pullStartY = pullMoveY = null;
    dist = distResisted = 0;
  }

  window.addEventListener('touchend', _onTouchEnd);
  window.addEventListener('touchstart', _onTouchStart);
  window.addEventListener('touchmove', _onTouchMove, { passive: false });

  // Store event handlers to use for teardown later
  return {
    onTouchStart: _onTouchStart,
    onTouchMove: _onTouchMove,
    onTouchEnd: _onTouchEnd
  };
}

function _run() {
  var _SETTINGS7 = _SETTINGS,
      mainElement = _SETTINGS7.mainElement,
      classPrefix = _SETTINGS7.classPrefix,
      onInit = _SETTINGS7.onInit,
      containerClassName = _SETTINGS7.containerClassName;


  if (!_SETTINGS.ptrElement) {
    var ptr = document.createElement('div');
    if (mainElement !== document.body) {
      mainElement.parentNode.insertBefore(ptr, mainElement);
    } else {
      document.body.insertBefore(ptr, document.body.firstChild);
    }

    ptr.classList.add(classPrefix + 'ptr');
    if (containerClassName !== '') {
      ptr.classList.add('' + containerClassName);
    }
    ptr.innerHTML = defaultHTML;
    _SETTINGS.ptrElement = ptr;
  }

  var styleEl = document.createElement('style');

  styleEl.textContent = defaultStyle;

  // document.head.appendChild(styleEl);

  document.head.insertBefore(styleEl, document.head.firstChild);

  if (typeof onInit === 'function') {
    onInit(_SETTINGS);
  }

  return {
    styleNode: styleEl,
    ptrElement: _SETTINGS.ptrElement
  };
}

var updateElement = function updateElement() {
  defaultStyle = '\n    .' + _SETTINGS.classPrefix + 'ptr {\n      background: #E0E0E0;\n      pointer-events: none;\n      font-size: 0.85em;\n      font-weight: bold;\n      top: 0;\n      height: 0;\n      transition: height 0.3s, min-height 0.3s;\n      text-align: center;\n      width: 100%;\n      overflow: hidden;\n      display: flex;\n      align-items: flex-end;\n      align-content: stretch;\n    }\n    .' + _SETTINGS.classPrefix + 'box {\n      padding: 10px;\n      flex-basis: 100%;\n    }\n    .' + _SETTINGS.classPrefix + 'pull {\n      transition: none;\n    }\n    .' + _SETTINGS.classPrefix + 'release .' + _SETTINGS.classPrefix + 'icon {\n      transform: rotate(180deg);\n    }\n  ';

  defaultHTML = '\n    <div class="' + _SETTINGS.classPrefix + 'box' + (_SETTINGS.boxClassName ? ' ' + _SETTINGS.boxClassName : '') + '">\n      <div class="' + _SETTINGS.classPrefix + 'content' + (_SETTINGS.contentClassName ? ' ' + _SETTINGS.contentClassName : '') + '">\n        <div class="' + _SETTINGS.classPrefix + 'text' + (_SETTINGS.textClassName ? ' ' + _SETTINGS.textClassName : '') + '"></div>\n      </div>\n    </div>\n  ';
};

var Pull = {
  init: function init() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var handlers = void 0;
    Object.keys(_defaults).forEach(function (key) {
      _SETTINGS[key] = options[key] || _defaults[key];
    });

    if (typeof _SETTINGS.mainElement === 'string') {
      _SETTINGS.mainElement = document.querySelector(_SETTINGS.mainElement);
    }

    if (typeof _SETTINGS.ptrElement === 'string') {
      _SETTINGS.ptrElement = document.querySelector(_SETTINGS.ptrElement);
    }

    if (typeof _SETTINGS.triggerElement === 'string') {
      _SETTINGS.triggerElement = document.querySelector(_SETTINGS.triggerElement);
    }

    updateElement();

    if (!_setup) {
      handlers = _setupEvents();
      _setup = true;
    }

    var _run2 = _run(),
        styleNode = _run2.styleNode,
        ptrElement = _run2.ptrElement;

    return {
      destroy: function destroy() {
        // Teardown event listeners
        window.removeEventListener('touchstart', handlers.onTouchStart);
        window.removeEventListener('touchend', handlers.onTouchEnd);
        window.removeEventListener('touchmove', handlers.onTouchMove);

        // Remove ptr element and style tag
        styleNode.parentNode.removeChild(styleNode);
        ptrElement.parentNode.removeChild(ptrElement);

        // Enable setupEvents to run again
        _setup = false;

        // null object references
        handlers = null;
        styleNode = null;
        ptrElement = null;
        _SETTINGS = {};
      }
    };
  }
};

module.exports = Pull;